#! /usr/bin/python
# -*- coding: utf-8 -*-

# Report - un programma per vedere la RAI senza Silverlight
#
# Copyright © 2009 Pietro Battiston <toobaz@email.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from __future__ import division
import urllib2, random, gtk, re, os, shutil, glib, gobject
from xml.dom.minidom import parseString
from itertools import count
from subprocess import Popen
from sys import argv
try:
    import gio
    GIO_AVAILABLE = True
except:
    GIO_AVAILABLE = False    

__version__ = '0.5'

print "Report %s" % __version__

URL_CANALI = 'http://www.rai.tv/dl/RaiTV/videoWall/PublishingBlock-5566288c-3d21-48dc-b3e2-af7fbe3b2af8.xml'
STUFF_DIR = '.'
RE_STREAM = re.compile('HREF="([^"]*)"')

def debug(self, *args):
    if '--debug' in argv:
        for arg in args:
            print arg,

class Ui(object):
    def __init__(self):
        self._builder = gtk.Builder()
        self._builder.add_from_string(stringa_glade)
    
    def __getattr__(self, attr_name):
        self.obj = self._builder.get_object(attr_name)
        return self.obj

class Stream(object):
    catalogo = {}
    crea_id = count()
    def __init__(self, xml, conf):
        self.id = self.crea_id.next()
        self.catalogo[self.id] = self
        self.conf = conf
        self.xml = xml
        self.pixbuf = False
        url_cont = xml.getElementsByTagName('url')[0]
        padre = xml.parentNode

        testi = padre.getElementsByTagName('textUnit')
        self.descrizione = ''
        for nodo_testo in testi:
            if nodo_testo.getAttribute('type') == 'Testo breve':
                testo = nodo_testo.getElementsByTagName('text')[0]
                self.descrizione = testo.childNodes[0].data
                break

        immagini = padre.getElementsByTagName('imageUnit')
        self.url_immagine = ''
        self.file_immagine = ''
        if immagini:
            immagine = immagini[0].getElementsByTagName('image')[0]
            self.url_immagine = 'http://www.rai.it%s' % immagine.childNodes[0].data
        self.url = url_cont.childNodes[0].data
        self.nome = xml.getAttribute('name')
        self.xml_type = xml.getAttribute('type')
        self.proc = None
        self.proc_reg = None
        
    def piglia_immagine(self, report):
        if self.pixbuf:
            report.ui.logo.set_from_pixbuf(self.pixbuf)
            return
        self.report = report
        if GIO_AVAILABLE:
            self.gio = gio.File(self.url_immagine)
            self.gio.read_async(self.pigliata)

    def pigliata(self, gdaemonfile, result):
        try:
            data_in = self.gio.read_finish(result).read()
        except glib.GError:
            return

        file_immagine = self.conf['tempdir'] + str(random.random())
        out = open(file_immagine, 'w')
        out.write(data_in)
        out.close()
        im = gtk.image_new_from_file(file_immagine)
        big_pixbuf = im.get_pixbuf()
        ratio = big_pixbuf.get_width() / big_pixbuf.get_height()
        dims = (100, int(100/ratio)) if ratio > 1 else (int(100*ratio), 100)
        self.pixbuf = big_pixbuf.scale_simple(dims[0], dims[1], gtk.gdk.INTERP_BILINEAR)

        if self.report.programma is self:
            self.report.ui.logo.set_from_pixbuf(self.pixbuf)
    
    def riga(self):
        return [self.nome, self.id]

    def trova_indirizzo(self):
        stream_request = urllib2.Request(self.url)
        stream_request.add_header('User-Agent', self.conf['useragent'])
        stream_request.add_header('viaurl:', 'www.rai.tv')
        opener = urllib2.build_opener()
        file_content = opener.open(stream_request).read()
        try:
            self.stream = RE_STREAM.search(file_content).groups()[0]
        except AttributeError:
            raise AttributeError, file_content

    def play(self):
        if self.recording():
            comando = self.conf['riproduttore']
            lung = os.path.getsize(self.filename)
            self.proc = Popen([comando, self.filename])
        else:
            self.trova_indirizzo()

            comando = self.conf['riproduttore']
            debug('Comando:',  " ".join(comando.split(' ') + [self.stream]))
            self.proc = Popen([comando, self.stream])

    def registra(self, filename):
        self.filename = filename
        self.trova_indirizzo()
        comando = self.conf['registratore']  % self.filename
        debug('Comando:', " ".join(comando.split(' ') + [self.stream]))
        self.proc_reg = Popen(comando.split(' ') + [self.stream])
        
    def stop(self):
        Popen(['kill', str(self.proc.pid)])

    def stop_reg(self):
        Popen(['kill', str(self.proc_reg.pid)])

    def playing(self):
        return self.proc and (self.proc.poll() == None)

    def recording(self):
        return self.proc_reg and (self.proc_reg.poll() == None)
        
class Canale(object):
    crea_id = count()
    def __init__(self, xml, conf):
        self.conf = conf
        self.id = self.crea_id.next()
        self.xml = xml
        self.nome = self.xml.getAttribute('name')
        self.streams = []
        for stream in xml.getElementsByTagName('videoUnit'):
            self.streams.append(Stream(stream, conf))
        
    def riga(self):
        return [self.nome, self.id]

class Configurazione(dict):
    def __init__(self, default):
        self.default = default
    
    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except:
            try:
                return self.default.__getitem__(item)
            except:
                raise KeyError, item
        

class Report(object):
    def __init__(self, tempdir=None):
        self.ui = Ui()
        if not tempdir:
            tempdir = '/tmp/%f/' % random.random()

        self.configurazione = Configurazione({'riproduttore' : 'mplayer',
                                              'registratore' : 'mplayer -dumpfile %s -dumpstream',
                                              'useragent' : 'Mozilla/5.0 (X11; U; Linux i686; it; rv:1.9.0.6) Gecko/2009011912 Firefox/3.0.11',
                                              'tempdir' : tempdir})


        self.ui.canaliview.insert_column_with_attributes(0, 'Nome', gtk.CellRendererText(), text=0)
        self.ui.programmiview.insert_column_with_attributes(0, 'Nome', gtk.CellRendererText(), text=0)

        self.selection = self.ui.canaliview.get_selection()
        self.selection.connect('changed', self.mostra_programmi)

        self.progr_selection = self.ui.programmiview.get_selection()
        self.progr_selection.connect('changed', self.mostra_info_programma)

        self.ui.programmiview.connect('row-activated', self.riproduci)
        
        self.ui.vai.connect('clicked', self.vai)
        self.ui.registra.connect('clicked', self.registra)
        
        glib.timeout_add(500, self.aggiorna_pulsanti)
        
    def run(self):
        self.ui.canali.append(['Sto caricando i canali. Attendere.', -1])

        if GIO_AVAILABLE:
            self.gio = gio.File(URL_CANALI)
            self.gio.read_async(self.lista_canali)
        else:
            gobject.idle_add(self.lista_canali)

        self.ui.main.run()

    def lista_canali(self, gdaemonfile=None, result=None, xml_canali=None):
        if result:
            xml_canali = self.gio.read_finish(result).read()
        else:
            # Manca GIO
            xml_canali=urllib2.urlopen(URL_CANALI).read()
            
        os.mkdir(self.configurazione['tempdir'])
#        xml_canali = open('PublishingBlock-5566288c-3d21-48dc-b3e2-af7fbe3b2af8.xml').read()
        doc = parseString(xml_canali)
        self.canali = {}
        for xml_set in doc.getElementsByTagName('set'):
            canale = Canale(xml_set, self.configurazione)
            self.canali[canale.id] = canale

        self.ui.canali.clear()
        for canale in self.canali.values():
            self.ui.canali.append(canale.riga())
        
        self.ui.canaliview.set_sensitive(True)
    
    def vai(self, *args):
        if self.programma.playing():
            self.programma.stop()
        else:
            self.programma.play()
        self.aggiorna_pulsanti()
    
    def registra(self, *args):
        if self.programma.recording():
            self.programma.stop_reg()
        else:
            if self.programma.playing():
                self.programma.stop()
            resp = self.ui.filechooser.run()
            self.ui.filechooser.hide()
            if resp == -4:
                return
            filename = self.ui.filechooser.get_filename()
            if not filename.endswith('.wmv'):
                filename += '.wmv'
            self.programma.registra(filename)
        self.aggiorna_pulsanti()
    
    def riproduci(self, tree, path, col):
        mod = tree.get_model()
        stream_id = mod[path[0]][1]
        stream = Stream.catalogo[stream_id]
        if not stream.playing():
            stream.play()
            self.aggiorna_pulsanti()
        else:
            print "already playing!"
    
    def mostra_programmi(self, selection=None):
        model, selection_iter = self.selection.get_selected()
        if not selection_iter:
            return
        self.ui.logo.set_from_pixbuf(None)
        self.ui.vai.set_sensitive(False)
        id_canale = model.get_value(selection_iter, 1)
        
        canale = self.canali[id_canale]
        self.ui.programmi.clear()
        for programma in canale.streams:
            self.ui.programmi.append(programma.riga())

    def mostra_info_programma(self, selection=None):
        model, selection_iter = self.progr_selection.get_selected()
        if not selection_iter:
            return
        self.ui.logo.set_from_pixbuf(None)
        id_programma = model.get_value(selection_iter, 1)

        self.programma = Stream.catalogo[id_programma]

        self.aggiorna_pulsanti()

        if self.programma.url_immagine:
            self.programma.piglia_immagine(self)

    def aggiorna_pulsanti(self):
        model, selection_iter = self.progr_selection.get_selected()
        if not selection_iter:
            self.ui.vai.set_label('Riproduci')
            self.ui.vai.set_sensitive(False)
            self.ui.registra.set_label('Registra')
            self.ui.registra.set_sensitive(False)
            return True
        
        self.ui.vai.set_sensitive(True)
        self.ui.registra.set_sensitive(True)

        if self.programma.playing():
            self.ui.vai.set_label('Ferma riproduzione')
        elif self.programma.recording():
            self.ui.vai.set_label('Vedi registrazione')
        else:
            self.ui.vai.set_label('Riproduci')
        
        if self.programma.recording():
            self.ui.registra.set_label('Ferma registrazione')
        else:
            self.ui.registra.set_label('Registra')

        return True

stringa_glade = """<?xml version="1.0"?>
<interface>
  <requires lib="gtk+" version="2.16"/>
  <!-- interface-naming-policy project-wide -->
  <object class="GtkListStore" id="canali">
    <columns>
      <!-- column-name Nome -->
      <column type="gchararray"/>
      <!-- column-name Id -->
      <column type="gint"/>
    </columns>
  </object>
  <object class="GtkListStore" id="programmi">
    <columns>
      <!-- column-name Nome -->
      <column type="gchararray"/>
      <!-- column-name Id -->
      <column type="gint"/>
    </columns>
  </object>
  <object class="GtkDialog" id="main">
    <property name="border_width">5</property>
    <property name="title" translatable="yes">Report (ed altro?!)</property>
    <property name="window_position">center</property>
    <property name="type_hint">normal</property>
    <property name="has_separator">False</property>
    <child internal-child="vbox">
      <object class="GtkVBox" id="dialog-vbox1">
        <property name="visible">True</property>
        <property name="orientation">vertical</property>
        <property name="spacing">2</property>
        <child>
          <object class="GtkHPaned" id="hpaned1">
            <property name="visible">True</property>
            <property name="can_focus">True</property>
            <child>
              <object class="GtkVBox" id="vbox1">
                <property name="visible">True</property>
                <property name="orientation">vertical</property>
                <child>
                  <object class="GtkLabel" id="label1">
                    <property name="visible">True</property>
                    <property name="label" translatable="yes">Canali:</property>
                  </object>
                  <packing>
                    <property name="expand">False</property>
                    <property name="fill">False</property>
                    <property name="position">0</property>
                  </packing>
                </child>
                <child>
                  <object class="GtkScrolledWindow" id="scrolledwindow1">
                    <property name="visible">True</property>
                    <property name="can_focus">True</property>
                    <property name="hscrollbar_policy">automatic</property>
                    <property name="vscrollbar_policy">automatic</property>
                    <child>
                      <object class="GtkTreeView" id="canaliview">
                        <property name="sensitive">False</property>
                        <property name="width_request">300</property>
                        <property name="height_request">300</property>
                        <property name="visible">True</property>
                        <property name="can_focus">True</property>
                        <property name="model">canali</property>
                      </object>
                    </child>
                  </object>
                  <packing>
                    <property name="position">1</property>
                  </packing>
                </child>
              </object>
              <packing>
                <property name="resize">False</property>
                <property name="shrink">True</property>
              </packing>
            </child>
            <child>
              <object class="GtkVBox" id="vbox2">
                <property name="visible">True</property>
                <property name="orientation">vertical</property>
                <child>
                  <object class="GtkHButtonBox" id="hbuttonbox1">
                    <property name="visible">True</property>
                    <property name="layout_style">spread</property>
                    <child>
                      <object class="GtkButton" id="vai">
                        <property name="visible">True</property>
                        <property name="sensitive">False</property>
                        <property name="can_focus">True</property>
                        <property name="receives_default">True</property>
                      </object>
                      <packing>
                        <property name="expand">False</property>
                        <property name="fill">False</property>
                        <property name="position">0</property>
                      </packing>
                    </child>
                    <child>
                      <object class="GtkButton" id="registra">
                        <property name="visible">True</property>
                        <property name="sensitive">False</property>
                        <property name="can_focus">True</property>
                        <property name="receives_default">True</property>
                      </object>
                      <packing>
                        <property name="expand">False</property>
                        <property name="fill">False</property>
                        <property name="position">1</property>
                      </packing>
                    </child>
                  </object>
                  <packing>
                    <property name="expand">False</property>
                    <property name="fill">False</property>
                    <property name="position">0</property>
                  </packing>
                </child>
                <child>
                  <object class="GtkScrolledWindow" id="scrolledwindow2">
                    <property name="visible">True</property>
                    <property name="can_focus">True</property>
                    <property name="hscrollbar_policy">automatic</property>
                    <property name="vscrollbar_policy">automatic</property>
                    <child>
                      <object class="GtkTreeView" id="programmiview">
                        <property name="width_request">300</property>
                        <property name="visible">True</property>
                        <property name="can_focus">True</property>
                        <property name="model">programmi</property>
                      </object>
                    </child>
                  </object>
                  <packing>
                    <property name="position">1</property>
                  </packing>
                </child>
                <child>
                  <object class="GtkHBox" id="hbox1">
                    <property name="visible">True</property>
                    <child>
                      <object class="GtkImage" id="logo">
                        <property name="visible">True</property>
                      </object>
                      <packing>
                        <property name="position">0</property>
                      </packing>
                    </child>
                    <child>
                      <object class="GtkLabel" id="descrizione">
                        <property name="visible">True</property>
                        <property name="wrap">True</property>
                      </object>
                      <packing>
                        <property name="position">1</property>
                      </packing>
                    </child>
                  </object>
                  <packing>
                    <property name="position">2</property>
                  </packing>
                </child>
              </object>
              <packing>
                <property name="resize">True</property>
                <property name="shrink">True</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="position">1</property>
          </packing>
        </child>
        <child internal-child="action_area">
          <object class="GtkHButtonBox" id="dialog-action_area1">
            <property name="visible">True</property>
            <property name="layout_style">end</property>
            <child>
              <placeholder/>
            </child>
            <child>
              <object class="GtkButton" id="button1">
                <property name="label">gtk-quit</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">True</property>
                <property name="use_stock">True</property>
              </object>
              <packing>
                <property name="expand">False</property>
                <property name="fill">False</property>
                <property name="position">1</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="pack_type">end</property>
            <property name="position">0</property>
          </packing>
        </child>
      </object>
    </child>
    <action-widgets>
      <action-widget response="0">button1</action-widget>
    </action-widgets>
  </object>
  <object class="GtkFileChooserDialog" id="filechooser">
    <property name="border_width">5</property>
    <property name="title" translatable="yes">Scegliere il nome del file</property>
    <property name="window_position">center</property>
    <property name="type_hint">normal</property>
    <property name="has_separator">False</property>
    <property name="action">save</property>
    <property name="do_overwrite_confirmation">True</property>
    <child internal-child="vbox">
      <object class="GtkVBox" id="dialog-vbox2">
        <property name="visible">True</property>
        <property name="orientation">vertical</property>
        <property name="spacing">2</property>
        <child>
          <placeholder/>
        </child>
        <child internal-child="action_area">
          <object class="GtkHButtonBox" id="dialog-action_area2">
            <property name="visible">True</property>
            <property name="layout_style">end</property>
            <child>
              <object class="GtkButton" id="button3">
                <property name="label">gtk-cancel</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">True</property>
                <property name="use_stock">True</property>
              </object>
              <packing>
                <property name="expand">False</property>
                <property name="fill">False</property>
                <property name="position">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkButton" id="button2">
                <property name="label">gtk-ok</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="can_default">True</property>
                <property name="has_default">True</property>
                <property name="receives_default">True</property>
                <property name="use_stock">True</property>
              </object>
              <packing>
                <property name="expand">False</property>
                <property name="fill">False</property>
                <property name="position">1</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="pack_type">end</property>
            <property name="position">0</property>
          </packing>
        </child>
      </object>
    </child>
    <action-widgets>
      <action-widget response="-4">button3</action-widget>
      <action-widget response="-5">button2</action-widget>
    </action-widgets>
  </object>
</interface>"""

if __name__ == '__main__':
    tempdir = '/tmp/%f/' % random.random()
    try:
        r = Report(tempdir)
        r.run()
    finally:
        shutil.rmtree(tempdir)
        for stream in Stream.catalogo.values():
            if stream.playing():
                stream.stop()
            if stream.recording():
                stream.stop_reg()
